Attribute VB_Name = "mdlACTType"
Option Explicit


' Contact table fields
Public Type udtContact
    UniqueID As String          ' CF_UniqueID
    Name As String              ' CF_Name
End Type

' Group table fields
Public Type udtGroup
    UniqueID As String          ' GF_UniqueID
    Name As String              ' GF_Name
End Type

' List table fields
Public Const TotalListFields = 7
Public Type udtList
    UniqueID As String          ' LTF_UniqueID = 1
    CreateTime As String        ' LTF_CreateTime = 2
    EditTime As String          ' LTF_EditTime = 3
    MergeTime As String         ' LTF_MergeTime = 4
    UserId As String            ' LTF_UserId = 6
    Type As String              ' LTF_Type = 25
    Name As String              ' LTF_Name = 26
End Type


' Sales table fields
Public Const TotalSalesFields = 23
Public Type udtSales
    UniqueID As String          ' SLF_UniqueID = 1
    CreateTime As String        ' SLF_CreateTime = 2
    EditTime As String          ' SLF_EditTime = 3
    MergeTime As String         ' SLF_MergeTime = 4
    UserId As String            ' SLF_UserId = 6
    ContactId As String         ' SLF_ContactId = 25
    GroupId As String           ' SLF_GroupId = 26
    Status As Integer           ' SLF_Status = 27
    ProductId As String         ' SLF_ProductId = 28
    TypeId As String            ' SLF_TypeId = 29
    Probability As Integer      ' SLF_Probability = 30
    SalesDate As String          ' SLF_SalesDate = 31
    Units As Long               ' SLF_Units = 32
    UnitPrice As Double         ' SLF_UnitPrice = 33
    Amount As Double            ' SLF_Amount = 34
    SaleStartDate As String    ' SLF_SaleStartDate = 35
    Notes As String             ' SLF_Notes = 36
    CompetitorsId As String     ' SLF_CompetitorsId = 37
    Reason As String            ' SLF_Reason = 38
    SalesStage As String        ' SLF_SalesStage = 39
    ProductName As String       ' SLVF_ProductName = 200
    TypeName As String          ' SLVF_TypeName = 201
    Competitors As String       ' SLVF_Competitors = 202
End Type



' Purpose: creates an array of List table field ids to be used with ACT!DatabaseObject.ListTable.GetDataEx()
' Note: the ids are sorted in increasing order
Public Sub ListIdArray(listIds() As Integer)
    ReDim listIds(1 To TotalListFields)
    Dim index As Integer
    index = LBound(listIds)
    listIds(index) = LTF_UniqueID
    listIds(index + 1) = LTF_CreateTime
    listIds(index + 2) = LTF_EditTime
    listIds(index + 3) = LTF_MergeTime
    listIds(index + 4) = LTF_UserId
    listIds(index + 5) = LTF_Type
    listIds(index + 6) = LTF_Name
End Sub

'get Popup list items for the specified field in Sales table
Public Function getSalesPopupList(objDb As Object, fieldId As Integer, items() As String) As Boolean
    Dim objSales As Object
    Dim objPopup As Object
    Dim bPopup As Boolean
    
    Set objSales = objDb.Sales
    
    bPopup = objSales.Fields.HasPopupList(fieldId)
    If bPopup Then
        Set objPopup = objSales.Fields.POPUPINFO(fieldId, 0)
    
        Dim nTotalItems As Integer
        nTotalItems = objPopup.PopupCount
        If (nTotalItems > 0) Then
            ReDim Preserve items(nTotalItems - 1) As String
        
            Dim i As Integer
            For i = 0 To nTotalItems - 1
                items(i) = objPopup.Value(i)
            Next
        Else
            bPopup = False
        End If
    
    End If
    getSalesPopupList = bPopup
End Function

'Purpose: copies data array into a Collection based on Ids
Public Sub ArrayToCollection(data() As String, Ids() As Integer, therecord As Collection)
    ReDim data(LBound(Ids) To UBound(Ids)) As String
    Set therecord = New Collection
    
    Dim index As Integer
    For index = LBound(Ids) To UBound(Ids)
        therecord.Add data(index), CStr(Ids(index))
    Next
    
End Sub

' Purpose: copies data array into a List Table record Collection
' Note:
'   * index is based on ListIdArray()
'   * the Collection items accessed by Field ID string (eg. cstr(LTF_UniqueID))
Public Sub listArrayToCollection(data() As String, listrecord As Collection)
    Dim index As Integer
    index = LBound(data)
    
    Set listrecord = New Collection
    listrecord.Add data(index), CStr(LTF_UniqueID)
    listrecord.Add data(index + 1), CStr(LTF_CreateTime)
    listrecord.Add data(index + 2), CStr(LTF_EditTime)
    listrecord.Add data(index + 3), CStr(LTF_MergeTime)
    listrecord.Add data(index + 4), CStr(LTF_UserId)
    listrecord.Add data(index + 5), CStr(LTF_Type)
    listrecord.Add data(index + 6), CStr(LTF_Name)
End Sub

Public Sub listArrayToUserType(data() As String, listtype As udtList)
    Dim index As Integer
    index = LBound(data)
    
    listtype.UniqueID = data(index)
    listtype.CreateTime = data(index + 1)
    listtype.EditTime = data(index + 2)
    listtype.MergeTime = data(index + 3)
    listtype.UserId = data(index + 4)
    listtype.Type = data(index + 5)
    listtype.Name = data(index + 6)
End Sub

'get product names, product types, and competitors from list table in database
Public Sub getListTable(objDb As Object, prodname() As Collection, prodtype() As Collection, competitor() As Collection)

    Dim objList As Object
    Set objList = objDb.ListTable
    Dim Ids() As Integer
    Dim data() As String
    
    ListIdArray Ids
    ReDim data(LBound(Ids) To UBound(Ids))
    
    Dim ip As Integer: ip = 0
    Dim it As Integer: it = 0
    Dim ic As Integer: ic = 0
    Dim listtype As udtList
    objList.MoveFirst
    Do While (Not objList.iseof)
        objList.GetDataEx Ids, data
        Select Case objList.data(LTF_Type)
        Case directorytype_product:
            ReDim Preserve prodname(ip)
            Set prodname(ip) = Nothing
            listArrayToCollection data, prodname(ip)
            ip = ip + 1
        Case directorytype_type:
            ReDim Preserve prodtype(it)
            Set prodtype(it) = Nothing
            listArrayToCollection data, prodtype(it)
            it = it + 1
        Case directorytype_maincompetitor:
            ReDim Preserve competitor(ic)
            Set competitor(ic) = Nothing
            listArrayToCollection data, competitor(ic)
            ic = ic + 1
        End Select
        objList.MoveNext
    Loop
    
End Sub

'uses Application Object to get the group list
Public Sub getGroupList(ByVal objapp As Object, grouplist() As udtGroup)
    Dim objGroup As Object
    Set objGroup = activateView(objapp, viewtype_groups, "GV")
    
    ReDim grouplist(objGroup.getCount - 1)
    
    Dim i As Integer
    objGroup.MoveFirst
    For i = LBound(grouplist) To UBound(grouplist)
        If (objGroup.isExpanded = False) Then
            objGroup.Expand
        End If
        grouplist(i).UniqueID = objGroup.GetField(GF_UniqueID)
        grouplist(i).Name = objGroup.GetField(GF_Name)
        objGroup.MoveNext
    Next i
End Sub

'create view ==> makes view active
Public Function activateView(ByVal objapp As Object, iType As Integer, szName As String, Optional iState As Integer = viewstate_maximized)
    Dim objViews As Object
    Dim objType As Object
    
    Set objViews = objapp.Views
    Set objType = objViews.CreateEx(iType, szName, iState)
    
    Set activateView = objType
End Function

Public Function shortDateStyle(thedate As String)
    Dim month As String, day As String, year As String
    year = Left$(thedate, 4)
    month = Mid$(thedate, 5, 2)
    day = Right$(thedate, 2)
    shortDateStyle = month & "/" & day & Right$(year, 2)
End Function


