// ActCmdExample.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "ActCmdExample.h"
#include "Resource.h"
#include <TCHAR.H>
#include <IO.H>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


///////////////////////////////////////////////////////////////////////////////
// This is the class that defines the methods used to call functions exported
// through OLE Automation by ActCmd.dll.  To keep things simple the methods are
// defined inside the class definition.

class IAuxCmdInterface : public COleDispatchDriver
{
public:
    long AddAuxCommand(LPCTSTR szCommandName, LPCTSTR szCommandLine,
                       LPCTSTR szStartInDir, LPCTSTR szToolTip,
                       LPCTSTR szCommandDescription, LPCTSTR szSmallIconPath,
                       LPCTSTR szLargeIconPath, short iRunState)
    {
        static BYTE parms[] = VTS_BSTR VTS_BSTR VTS_BSTR VTS_BSTR VTS_BSTR VTS_BSTR VTS_BSTR VTS_I2;
        long        lRet;
        
        InvokeHelper(0x01, DISPATCH_METHOD, VT_I4, &lRet, parms,
                     szCommandName, szCommandLine, szStartInDir, szToolTip,
                     szCommandDescription, szSmallIconPath,
                     szLargeIconPath, iRunState);
        return lRet;
    }
    
    long DeleteAuxCommand(LPCTSTR szCommandName)
    {
        static BYTE parms[] = VTS_BSTR;
        long        lRet;
        
        InvokeHelper(0x02, DISPATCH_METHOD, VT_I4, &lRet, parms, szCommandName);
        return lRet;
    }
    
    BOOL AuxCommandExists(LPCTSTR szCommandName)
    {
        static BYTE parms[] = VTS_BSTR;
        BOOL        bRet;
        
        InvokeHelper(0x03, DISPATCH_METHOD, VT_I4, &bRet, parms, szCommandName);
        return bRet;
    }
    
    long AddAuxCommandToToolsMenu(long iViewID, LPCTSTR szCommandName)
    {
        static BYTE parms[] = VTS_I4 VTS_BSTR;
        long        lRet;
        
        InvokeHelper(0x04, DISPATCH_METHOD, VT_I4, &lRet, parms, iViewID, szCommandName);
        return lRet;
    }
    
    long RemoveAuxCommandFromToolsMenu(long iViewID, LPCTSTR szCommandName)
    {
        static BYTE parms[] = VTS_I4 VTS_BSTR;
        long        lRet;
        
        InvokeHelper(0x05, DISPATCH_METHOD, VT_I4, &lRet, parms, iViewID, szCommandName);
        return lRet;
    }
    
    BOOL AuxCommandExistsInToolsMenu(long iViewID, LPCTSTR szCommandName)
    {
        static BYTE parms[] = VTS_I4 VTS_BSTR;
        BOOL        bRet;
        
        InvokeHelper(0x06, DISPATCH_METHOD, VT_I4, &bRet, parms, iViewID, szCommandName);
        return bRet;
    }
    
    long AddAuxCommandToToolBar(long iViewID, LPCTSTR szCommandName)
    {
        static BYTE parms[] = VTS_I4 VTS_BSTR;
        long        lRet;
        
        InvokeHelper(0x07, DISPATCH_METHOD, VT_I4, &lRet, parms, iViewID, szCommandName);
        return lRet;
    }
    
    long RemoveAuxCommandFromToolBar(long iViewID, LPCTSTR szCommandName)
    {
        static BYTE parms[] = VTS_I4 VTS_BSTR;
        long        lRet;
        
        InvokeHelper(0x08, DISPATCH_METHOD, VT_I4, &lRet, parms, iViewID, szCommandName);
        return lRet;
    }
    
    BOOL AuxCommandExistsInToolBar(long iViewID, LPCTSTR szCommandName)
    {
        static BYTE parms[] = VTS_I4 VTS_BSTR;
        BOOL        bRet;
        
        InvokeHelper(0x09, DISPATCH_METHOD, VT_I4, &bRet, parms, iViewID, szCommandName);
        return bRet;
    }
};

/////////////////////////////////////////////////////////////////////////////
// The one and only CActCmdExampleApp object

CActCmdExampleApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CActCmdExampleApp initialization

// We are just going to do our SDK thing with aux commands and
// then exit - so all the SDK commands are called during InitInstance
// and then	we will return FALSE so the app will immediately exit.

BOOL CActCmdExampleApp::InitInstance()
{
#ifdef _AFXDLL
    Enable3dControls();			// Call this when using MFC in a shared DLL
#else
    Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif
    
    if (!AfxOleInit())
    {
        AfxMessageBox(IDP_OLE_INIT_FAILED);
        return FALSE;
    }
    
    // get users OK to go ahead
    if ( AfxMessageBox(_T("Click OK to go ahead with install of toolbar and menu links to NotePad.exe and Clock.exe"),
                       MB_OKCANCEL) != IDOK)
    {
        return FALSE;
    }
    
    IAuxCmdInterface	AuxCmds;
    long				lRet;
    BOOL				bRet;
    
    // connect to the AUXCMDS server
    if ( !AuxCmds.CreateDispatch(_T("ACTOLE.AUXCMDS") ) )
    {
        AfxMessageBox( "Unable to load ActCmd.dll. May be missing or not registered" );
        return FALSE;
    }

    // Get ACT! path from user.  This will be used to make paths to toolbar icons.
    CString sPath;

    CPathDlg getPathDlg;
    getPathDlg.SetStaticMsg(_T("Enter the path to use for tool bar icons:"));
    getPathDlg.SetDefaultPath(_T("C:\\Program Files\\Symantec\\Act"));
    if ( getPathDlg.DoModal(&sPath) != IDOK )
        return FALSE;
    
    // Make paths to icons by adding icon file names to ACT! path
    CString sSmallIconPath;
    CString sLargeIconPath;

    // Copy Act path
    sSmallIconPath = sPath;
    sLargeIconPath = sPath;

    // Add backslash if necessary
    if ( sSmallIconPath.Right(1) != _T("\\") ) sSmallIconPath += _T("\\");
    if ( sLargeIconPath.Right(1) != _T("\\") ) sLargeIconPath += _T("\\");

    sSmallIconPath += _T("strch_s1.ico");
    sLargeIconPath += _T("strch_l1.ico");

    // NotePad is in the windows folder - make a path to it
    TCHAR	szFilePath[MAX_PATH+1];
    CString sDirectory;
    CString sExePath;
    
    // get the path to load from 
    GetWindowsDirectory(szFilePath, MAX_PATH);
    sDirectory = szFilePath;

    // copy to executable path and add backslash if necessary
    sExePath = sDirectory;
    if ( sExePath.Right(1) != _T("\\") ) sExePath += _T("\\");

    // Add "NotePad.exe" to executable path
    sExePath += _T("NotePad.exe");
        
    // Add a command definition.  Once the command is defined it can then be added to
    // an ACT! toolbar or menu for a specified view.  
    lRet = AuxCmds.AddAuxCommand(_T("Notepad"), sExePath, sDirectory,
                                 _T("Notepad"), _T("Notepad text editor"),
                                 sSmallIconPath, sLargeIconPath, 0 );
    
    // Clock is in the windows system folder - make a path to it
    GetSystemDirectory(szFilePath, MAX_PATH);
    sDirectory = szFilePath;

    // copy to executable path and add backslash if necessary
    sExePath = sDirectory;
    if ( sExePath.Right(1) != _T("\\") ) sExePath += _T("\\");

    // Add "NotePad.exe" to executable path
    sExePath += _T("Clock.exe");
        
    // Add another command this time small and large icon paths are NULL. This 
    // command can only be placed on a menu.
    lRet = AuxCmds.AddAuxCommand(_T("Clock"), sExePath, sDirectory,
                                 _T("Clock"), _T("The Clock"), NULL, NULL, 0 );
    
    // Check to see if "Clock" has been added
    bRet = AuxCmds.AuxCommandExists(_T("Clock"));
    if (!bRet)
    {
        AfxMessageBox("The Clock program definition was not properly added");
    }
    
    // Add a button for the Notepad command to the contact view toolbar
    // Note: 1 = Contact View, 2 = Contact List, 3 = Group View,
    //       4 = Task List, 5 = Calendar View, 6 = Email View
    lRet = AuxCmds.AddAuxCommandToToolBar(1, _T("Notepad"));
    
    // Add a menu item the Tools menu in the Group view for "Clock"
    lRet = AuxCmds.AddAuxCommandToToolsMenu(3, _T("Clock"));
    
    return FALSE;	// return FALSE so application will quit
}


//////////////////////////////////////////////////////////////////////////
// Implementation of CPathDlg, a dialog that gets a directory path from user


CPathDlg::CPathDlg(CWnd* pParent /*=NULL*/)
: CDialog(IDD_PATH_DIALOG, pParent)
{
    m_pSPath = NULL;
}

CPathDlg::~CPathDlg()
{
}

BEGIN_MESSAGE_MAP(CPathDlg, CDialog)
//{{AFX_MSG_MAP(CPathDlg)
ON_WM_CLOSE()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPathDlg message handlers

BOOL CPathDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    CWnd    *pCWnd;

    if (pCWnd = GetDlgItem(IDC_STATIC_MSG))
        pCWnd->SetWindowText(m_sStaticMsg);


    if (pCWnd = GetDlgItem(IDC_EDIT_PATH))
    {
        // set default text and focus
        pCWnd->SetWindowText(m_sDefaultPath);
        pCWnd->SetFocus();
    }

    return FALSE;  // return FALSE if focus to a control
}

void CPathDlg::SetStaticMsg(LPCTSTR lpcMsg)
{
    m_sStaticMsg = lpcMsg;
}

void CPathDlg::SetDefaultPath(LPCTSTR lpcDefPath)
{
    m_sDefaultPath = lpcDefPath;
}

int CPathDlg::DoModal(CString *pSPath)
{
    m_pSPath = pSPath;
    return CDialog::DoModal();
}

void CPathDlg::OnOK() 
{
    if ( m_pSPath != NULL )
    {
        CWnd    *pCWnd;
        if (pCWnd = GetDlgItem(IDC_EDIT_PATH))
        {
            pCWnd->GetWindowText(*m_pSPath);
            if ( !m_pSPath->IsEmpty() )
            {
                // check validity of directory path
                if ( _taccess(*m_pSPath, 0) != 0 )
                {
                    AfxMessageBox(_T("Path does not exist"));
                    if (pCWnd = GetDlgItem(IDC_EDIT_PATH)) pCWnd->SetFocus();
                    return;
                }
            }
        }
    }

    CDialog::OnOK();
}

void CPathDlg::OnCancel() 
{
     CDialog::OnCancel();
}

